package paraphylo.solver.cplex;

import ilog.concert.IloException;
import ilog.concert.IloIntExpr;
import ilog.concert.IloIntVar;
import ilog.concert.IloNumExpr;
import ilog.cplex.IloCplex;
import ilog.cplex.IloCplex.UnknownObjectException;
import paraphylo.dao.Parameters;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;
import paraphylo.util.NullOutputStream;


public class CplexLeastResolvedTreeSolver extends CplexAbstractSolver{
	
	// map variables
	private IloIntVar[][] M;
	// cluster variables
	private IloIntVar[][][] N;
	// gamete variables
	private IloIntVar[][][] C;
	// non-trivial columns of M
	private IloIntVar[] Y;
	
	private int timelimit;
	
	public CplexLeastResolvedTreeSolver(MultiGeneFamilySpeciesDataSet ds, Parameters parameters) throws IloException{
		super(ds, parameters);
		this.timelimit = parameters.getTimelimitLRT();
	}
	
	public String solve(int lowerBound, int upperBound) throws IloException{
		this.solver.setName("CplexLeastResolvedTreeSolver");

		// initialize variables M(a,p)
		this.M = new IloIntVar[noOfSpecies][noOfSpecies-2];
		for (int a=0; a<noOfSpecies; a++){
			this.M[a] = solver.boolVarArray(noOfSpecies-2);
			for (int p=0; p<noOfSpecies-2; p++){
				M[a][p].setName("M_("+a+","+p+")");
			}
		}
		
		// initialize variables N((ab|c),p)
		this.N = new IloIntVar[noOfSpecies][noOfSpecies][noOfSpecies-2];
		for (int a=0; a<noOfSpecies; a++){
			for (int b=a+1; b<noOfSpecies; b++){
				this.N[a][b] = solver.boolVarArray(noOfSpecies-2);
				for (int p=0; p<noOfSpecies-2; p++){
					N[a][b][p].setName("N_("+a+","+b+"),"+p);
				}
			}
		}
		
		
		// 0 <= − 2 ∗ N((ab), p) + M(a, p) + M(b, p) <= 1
		for (int p=0; p<this.noOfSpecies-2;p++){
			for (int a=0; a<noOfSpecies; a++){
				for (int b=a+1; b<noOfSpecies; b++){
					IloIntExpr expr = solver.sum(solver.prod(-2, N[a][b][p]),M[a][p],M[b][p]);
					solver.addRange(0, expr, 1);
//					solver.addLazyConstraint(solver.range(0, expr, 1));
				}
			}
		}
		
		// for all T(ab|c)==1 : 1 <= SUM_p( N((ab),p) - 0.5*N((ac),p) - 0.5*N((bc),p))
		for (int a=0; a<noOfSpecies; a++){
			for (int b=a+1; b<noOfSpecies; b++){
				for (int c=0; c<noOfSpecies; c++){
					if (a!=c && b!=c && ds.getConsistentTriple(a, b, c)){
						IloNumExpr expr = solver.sum(solver.sum(N[a][b]),solver.prod(-0.5,solver.sum(N[a<c?a:c][a<c?c:a])),solver.prod(-0.5,solver.sum(N[b<c?b:c][b<c?c:b])));
						solver.addLe(1, expr);
					}
				}
			}
		}
		
		// initialize variables C(p,q, xy)
		this.C = new IloIntVar[noOfSpecies-2][noOfSpecies-2][3];
		for (int p=0; p<noOfSpecies-2; p++){
			for (int q=p+1; q<noOfSpecies-2; q++){
				this.C[p][q] = solver.boolVarArray(3);
			}
		}
		
		// C(p,q,01) >= − M(a,p) + M(a,q)
		// C(p,q,10) >= M(a,p) − M(a,q)
		// C(p,q,11) >= M(a,p) + M(a,q) − 1
		for (int a=0; a<noOfSpecies; a++){
			for (int p=0; p<noOfSpecies-2; p++){
				for (int q=p+1; q<noOfSpecies-2; q++){
					IloIntExpr expr1 = solver.sum(solver.negative(M[a][p]),M[a][q]);
					solver.addGe(C[p][q][0], expr1);
					IloIntExpr expr2 = solver.sum(M[a][p], solver.negative(M[a][q]));
					solver.addGe(C[p][q][1], expr2);
					IloIntExpr expr3 = solver.sum(M[a][p],M[a][q],solver.constant(-1));
					solver.addGe(C[p][q][2], expr3);
				}
			}
		}
		
		// C(p,q,01) + C(p,q,10) + C(p,q,11) <= 2
		for (int p=0; p<noOfSpecies-2; p++){
			for (int q=p+1; q<noOfSpecies-2; q++){
				solver.addLe(solver.sum(C[p][q]), 2);
			}
		}
		
		// initialize variables Y_p
		this.Y = solver.boolVarArray(noOfSpecies-2);

		// 0 <= Yp*|S| − SUM_A(M(A,p)) <= |S|−1
		for (int p=0; p<noOfSpecies-2; p++){
			IloIntExpr expr = solver.constant(0);
			for (int a=0; a<noOfSpecies; a++){
				expr = solver.sum(expr,M[a][p]);
			}
			expr = solver.sum(solver.prod(Y[p],noOfSpecies),solver.negative(expr));
			solver.addRange(0, expr, noOfSpecies-1);
		}
		
		// sort constraints
		for (int p=0; p<noOfSpecies-3; p++){
			IloNumExpr expr1 = solver.constant(0);
			IloNumExpr expr2 = solver.constant(0);
			for (int a=0; a<noOfSpecies; a++){
				long factor = 1 << ((noOfSpecies-1)-a);
				expr1 = solver.sum(expr1, solver.prod(factor, M[a][p]));
				expr2 = solver.sum(expr2, solver.prod(factor, M[a][p+1]));
			}
			solver.addGe(expr1, solver.sum(expr2, Y[p]));
		}
		
		this.objFn = solver.sum(Y);
		solver.addRange(lowerBound, this.objFn, upperBound);
		solver.addMinimize(this.objFn);

		if (this.timelimit>0){
			solver.setParam(IloCplex.DoubleParam.TiLim, timelimit);
		}

		// solve
		if (!parameters.isVerbose()){
			solver.setOut(new NullOutputStream());
		}
		boolean ret = solver.solve();
		String solution = "none";
		if (ret) {
			solution = getSolution();
		}
		// free memory
		solver.end();
		
		return solution;

	}
	
	/**
	 * @return the current solution, i.e. the values of xVariables as string
	 * @throws UnknownObjectException
	 * @throws IloException
	 */
	private String getSolution() throws UnknownObjectException, IloException {
		String solution="Map:\n";
		boolean[][] hirarchy = new boolean[noOfSpecies][this.noOfSpecies-2];
		for (int a=0; a<this.noOfSpecies;a++){
			double variable[] = solver.getValues(this.M[a]);
			for (int p=0; p<this.noOfSpecies-2; p++){
				solution+=(variable[p]<0.5?0:1)+" ";
				hirarchy[a][p]=variable[p]>0.5;
			}
			solution+="\n";
		}
		ds.setHirarchy(hirarchy);
		solution+="\n\n";
		solution+=solver.getObjValue()+"\n";
		return solution;
	}
	
}