package paraphylo.solver.cplex;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import paraphylo.dao.Parameters;
import paraphylo.dao.Triple;
import paraphylo.dataset.GeneFamily;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;
import paraphylo.solver.greedy.GreedyConsistentSubsetSolver;
import paraphylo.util.BuildAlgorithm;
import paraphylo.util.TripleExtraction;
import paraphylo.util.Utils;

public class CplexPipeline {
	
	public static void solve(MultiGeneFamilySpeciesDataSet ds, Parameters parameters) throws IOException{
		long totalTime = System.currentTimeMillis();

		String output = parameters.getOutput();
		BufferedWriter bw = new BufferedWriter(new FileWriter(output));
		
		if (parameters.getBootstrap()==Parameters.NO_BOOTSTRAPPING){
			writeTrees(ds, parameters, false);
		}
		
		if (parameters.getInputType()==Parameters.DEFAULT_INPUT){
			solveCGE(ds, parameters, bw);
			if (parameters.isWrite()){
				System.out.println("Writing cographs");
				writeFile(ds.getCographOutput(), output+".cographs", false);
			}
		}
		
		//TODO uncomment for cograph editing benchmark
		if (parameters.getBootstrap()==Parameters.NO_BOOTSTRAPPING){
			solveNoBootstrap(ds, parameters, bw);
		}
		else if (parameters.getBootstrap()==Parameters.COGRAPH_BOOTSTRAPPING){
			solveCographBootstrap(ds, parameters, bw);
		}
		else if (parameters.getBootstrap()==Parameters.TRIPLE_BOOTSTRAPPING){
			solveTripleBootstrap(ds, parameters, bw);
		}
		
		bw.write("#total time\n");
		bw.write(String.valueOf((System.currentTimeMillis()-totalTime))+"\n");
		bw.close();
		
		
		
	}

	private static void solveNoBootstrap(MultiGeneFamilySpeciesDataSet ds,
			Parameters parameters, BufferedWriter bw) throws IOException {
		
		String output = parameters.getOutput();
		
		// extract triples
		if (parameters.getInputType()==Parameters.DEFAULT_INPUT || parameters.getInputType()==Parameters.COGRAPH_INPUT){
			extractTriples(ds, parameters, bw);
			// enrich triples
			if (parameters.getEnrichment()>0d){
				enrichTriples(ds, parameters, bw);
			}
			if (parameters.isWrite()){
				System.out.println("Writing triples");
				writeFile(ds.getTripleOutput(), output+".triples", false);
			}
		}
		
		if (ds.getNoOfDistinctTriples()!=0){
			solveMCSLRT(ds, parameters, bw, true);
		}
		
		writeTrees(ds, parameters, false);
		writeInfo(ds, parameters, bw);
	}

	private static void solveCographBootstrap(MultiGeneFamilySpeciesDataSet ds,
			Parameters parameters, BufferedWriter bw) throws IOException {
		
		String output = parameters.getOutput();
		
		// extract triples
		if (parameters.getInputType()==Parameters.DEFAULT_INPUT || parameters.getInputType()==Parameters.COGRAPH_INPUT){
			extractTriples(ds, parameters, bw);
			if (parameters.getEnrichment()>0d){
				enrichTriples(ds, parameters, bw);
			}
			if (parameters.isWrite()){
				System.out.println("Writing triples");
				writeFile(ds.getTripleOutput(), output+".triples", false);
			}
		}
		
		for (int i=1; i<=parameters.getReplications(); i++){
			System.out.println("Running cograph-based bootstrapping "+i+"/"+parameters.getReplications());
			MultiGeneFamilySpeciesDataSet rds = new MultiGeneFamilySpeciesDataSet(ds.getNoOfSpecies(), ds.getSpeciesNames());
			rds.setOriginalTree(ds.getOriginalTree());
			int noGf = ds.getGeneFamilies().size();
			for (int j=0; j<noGf; j++){
				GeneFamily rgf = ds.getGeneFamilies().get(parameters.getRandom().nextInt(noGf));
				for (Triple triple : rgf.getTriples()){
					rds.addTriple(triple.getA(), triple.getB(), triple.getC());
				}
			}
			
			if (rds.getNoOfDistinctTriples()!=0){
				solveMCSLRT(rds, parameters, bw, false);
			}
			
			writeTrees(rds, parameters, i>1);
			writeInfo(rds, parameters, bw);
		}
		
	}
	
	private static void solveTripleBootstrap(MultiGeneFamilySpeciesDataSet ds,
			Parameters parameters, BufferedWriter bw) throws IOException {
		
		String output = parameters.getOutput();
		
		// extract triples
		if (parameters.getInputType()==Parameters.DEFAULT_INPUT || parameters.getInputType()==Parameters.COGRAPH_INPUT){
			extractTriples(ds, parameters, bw);
			if (parameters.getEnrichment()>0d){
				enrichTriples(ds, parameters, bw);
			}
			if (parameters.isWrite()){
				System.out.println("Writing triples");
				writeFile(ds.getTripleOutput(), output+".triples", false);
			}
		}
		
		List<Triple> triples = ds.getTripleList();
		for (int i=1; i<=parameters.getReplications(); i++){
			System.out.println("Running triple-based bootstrapping "+i+"/"+parameters.getReplications());
			MultiGeneFamilySpeciesDataSet rds = new MultiGeneFamilySpeciesDataSet(ds.getNoOfSpecies(), ds.getSpeciesNames());
			rds.setOriginalTree(ds.getOriginalTree());
			int noTriples = triples.size();
			if (noTriples!=0){
				for (int j=0; j<noTriples; j++){
					Triple triple = triples.get(parameters.getRandom().nextInt(noTriples));
					rds.addTriple(triple.getA(), triple.getB(), triple.getC());
				}
			
				solveMCSLRT(rds, parameters, bw, false);
			}
			
			writeTrees(rds, parameters, i>1);
			writeInfo(rds, parameters, bw);
		}
		
	}
	
	public static void solveCGE(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw) throws IOException{
		// compute minimal cograph editing
		long time = System.currentTimeMillis();
		int counter = 0;
		int noGf = ds.getGeneFamilies().size();
		Iterator<GeneFamily> geneFamilyIterator = ds.getGeneFamilyIterator();
		while (geneFamilyIterator.hasNext()){
			counter++;
			GeneFamily gf = geneFamilyIterator.next();
			if (Utils.isCograph(gf) && Utils.checkOrthology(gf)){
				int noGenes = gf.getNoOfGenes();
				double[][] orthology = gf.getOrthologyMatrix();
				Boolean[][] cograph = new Boolean[noGenes][noGenes];
				for (int a=0; a<noGenes; a++){
					for (int b=0; b<noGenes; b++){
						cograph[a][b]=(orthology[a][b]>0.5);
					}
				}
				gf.addCograph(cograph);
			}
			else if (gf.getNoOfSpeciesInGeneFamily()>=3 ){
				String coGraphSolution = "none";
				try {
					CplexCographEditingSolver cographEditingSolver = new CplexCographEditingSolver(ds, gf, parameters);
					System.out.println("Running CographEditingSolver for component " + counter + "/" + noGf);
					coGraphSolution = cographEditingSolver.solve();
					if (parameters.isVerbose()){
						printSolution(coGraphSolution);
					}
				} catch (Exception e) {
					System.out.println("Warning: Could not compute closest cograph for gene family " + counter);
					if (parameters.isVerbose()){
						e.printStackTrace();
					}
				}
			}
			//TODO edit for cograph editing benchmark
//			System.out.println("Component: " + counter);
//			if (Utils.isCograph(gf)){
//				int noGenes = gf.getNoOfGenes();
//				double[][] orthology = gf.getOrthologyMatrix();
//				Boolean[][] cograph = new Boolean[noGenes][noGenes];
//				for (int a=0; a<noGenes; a++){
//					for (int b=0; b<noGenes; b++){
//						cograph[a][b]=(orthology[a][b]>0.5);
//					}
//				}
//				gf.addCograph(cograph);
//				System.out.println(0);
//			}
//			else {
//				String coGraphSolution = "none";
//				try {
//					CplexCographEditingSolver cographEditingSolver = new CplexCographEditingSolver(ds, gf, parameters);
//					coGraphSolution = cographEditingSolver.solve();
//					System.out.println(gf.getCographEditDistance());
//					if (parameters.isVerbose()){
//						printSolution(coGraphSolution);
//					}
//				} catch (Exception e) {
//					System.out.println("Warning: Could not compute closest cograph for gene family " + counter);
//					if (parameters.isVerbose()){
//						e.printStackTrace();
//					}
//				}
//			}
		}
		bw.write("#cograph solving time ("+counter+" gene families)\n");
		bw.write(String.valueOf((System.currentTimeMillis()-time))+"\n");
		bw.flush();
	}
	
	public static void extractTriples(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw) throws IOException{
		// extract triples from cographs
		long time = System.currentTimeMillis();
		int counter = 0;
		int noGf = ds.getGeneFamilies().size();
		Iterator<GeneFamily> geneFamilyIterator = ds.getGeneFamilyIterator();
		while (geneFamilyIterator.hasNext()){
			counter++;
			GeneFamily gf = geneFamilyIterator.next();
			if (gf.getNoOfSpeciesInGeneFamily()>=3 ){
				Iterator<Boolean[][]> cographIterator = gf.getCographs().iterator();
				while (cographIterator.hasNext()){
					Boolean[][] cograph = cographIterator.next();
					if (Utils.existsInterSpeciesParalogyInCograph(gf, cograph)){
						try {
							String tripleExtractionSolution = "";
							if (parameters.isQuickTE()){
								TripleExtraction tripleExtraction = new TripleExtraction(ds, gf, cograph, parameters);
								System.out.println("Running quick TripleExtraction for component " + counter + "/" + noGf);
								tripleExtractionSolution = tripleExtraction.solve();
							}
							else {
								CplexTripleExtractionSolver tripleExtractionSolver = new CplexTripleExtractionSolver(ds, gf, cograph, parameters);
								System.out.println("Running TripleExtractionSolver for component " + counter + "/" + noGf);
								tripleExtractionSolution = tripleExtractionSolver.solve();
							}
							if (parameters.isVerbose()){
								printSolution(tripleExtractionSolution);
							}
						} catch (Exception e) {
							System.out.println("Warning: Could not extract triples from gene family " + counter);
							if (parameters.isVerbose()){
								e.printStackTrace();
							}
						}
					}
				}
			}
		}
		bw.write("#triple extraction time ("+counter+" gene families)\n");
		bw.write(String.valueOf((System.currentTimeMillis()-time))+"\n");
		bw.flush();
	}
	
	public static void enrichTriples(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw) throws IOException{
		// extract triples from cographs
		long time = System.currentTimeMillis();
		int counter = 0;
		int noSpecies = ds.getNoOfSpecies();
		Iterator<GeneFamily> geneFamilyIterator = ds.getGeneFamilyIterator();
		System.out.println("Running Triple Enrichment");
		while (geneFamilyIterator.hasNext()){
			GeneFamily gf = geneFamilyIterator.next();
			if (gf.getBitscores()!=null){
				counter++;
				if (gf.getNoOfSpeciesInGeneFamily()>=3 ){
					Iterator<Boolean[][]> cographIterator = gf.getCographs().iterator();
					while (cographIterator.hasNext()){
						Boolean[][] cograph = cographIterator.next();
						boolean[][][] enrichedTriples = new boolean[noSpecies][noSpecies][noSpecies];
						for (int a=0; a<noSpecies; a++){
							for (int b=a+1; b<noSpecies; b++){
								for (int c=b+1; c<noSpecies; c++){
									List<Integer> genesX = gf.getGenesOfSpecies(a);
									for (int x : genesX){
										List<Integer> genesY = gf.getGenesOfSpecies(b);
										for (int y : genesY){
											List<Integer> genesZ = gf.getGenesOfSpecies(c);
											for (int z : genesZ){
												if (cograph[x][y] && cograph[x][z] && cograph[y][z]){
													double bitscoreXY = (gf.getBitscores()[x][y]+gf.getBitscores()[y][x])/2d;
													double bitscoreXZ = (gf.getBitscores()[x][z]+gf.getBitscores()[z][x])/2d;
													double bitscoreYZ = (gf.getBitscores()[y][z]+gf.getBitscores()[z][y])/2d;
													if (bitscoreXY*parameters.getEnrichment()>bitscoreXZ && bitscoreXY*parameters.getEnrichment()>bitscoreYZ){
														if (bitscoreXZ*parameters.getEnrichment()<bitscoreYZ && bitscoreYZ*parameters.getEnrichment()<bitscoreXZ){
															enrichedTriples[a][b][c] = true;
														}
													}
													if (bitscoreXZ*parameters.getEnrichment()>bitscoreXY && bitscoreXZ*parameters.getEnrichment()>bitscoreYZ){
														if (bitscoreXY*parameters.getEnrichment()<bitscoreYZ && bitscoreYZ*parameters.getEnrichment()<bitscoreXY){
															enrichedTriples[a][c][b] = true;;
														}
													}
													if (bitscoreYZ*parameters.getEnrichment()>bitscoreXY && bitscoreYZ*parameters.getEnrichment()>bitscoreXZ){
														if (bitscoreXY*parameters.getEnrichment()<bitscoreXZ && bitscoreXZ*parameters.getEnrichment()<bitscoreXY){
															enrichedTriples[b][c][a] = true;
														}
													}
												}
											}
										}
									}
								}
							}
						}
						for (int a=0; a<noSpecies; a++){
							for (int b=a+1; b<noSpecies; b++){
								for (int c=b+1; c<noSpecies; c++){
									if (enrichedTriples[a][b][c]){
										ds.addEnrichedTriple(a, b, c);
									}
									if (enrichedTriples[a][c][b]){
										ds.addEnrichedTriple(a, c, b);
									}
									if (enrichedTriples[b][c][a]){
										ds.addEnrichedTriple(b, c, a);
									}
								}
							}
						}
					}
				}
			}
		}
		double noTriples = ds.getNoOfTriples();
		double noEnrichedTriples = ds.getNoOfEnrichedTriples();
		if (noTriples>0 && noEnrichedTriples>0){
			double enrichmentFactor = parameters.getEnrichmentFactor();
			for (int a=0; a<noSpecies; a++){
				for (int b=a+1; b<noSpecies; b++){
					for (int c=0; c<noSpecies; c++){
						if (a!=c && b!=c){
							double value = ds.getTriples()[a][b][c]+(enrichmentFactor*ds.getEnrichedTriples()[a][b][c]);
							ds.getTriples()[a][b][c]=value;
							ds.getTriples()[b][a][c]=value;
						}
					}
				}
			}
		}
		bw.write("#triple enrichment time ("+counter+" gene families)\n");
		bw.write(String.valueOf((System.currentTimeMillis()-time))+"\n");
		bw.flush();
	}
	
	public static void solveMCSLRT(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw, boolean writeTreesImmediately) throws IOException{
		
		// compute maximal consistent subset
		if (!parameters.isGreedyCS()){
			solveMCS(ds, parameters, bw);
		}
		else{
			greedyCS(ds, parameters, bw);
		}
		// compute closure
		ds.closeConsistentTriples();
		// compute BUILD tree
		ds.setBuildTree(computeBuildTree(ds, parameters, bw));
		ds.setLrTree(ds.getBuildTree());
		if (writeTreesImmediately){
			writeTrees(ds, parameters, false);
		}
	
		int noInternalNodes = Utils.countOccurrences(ds.getBuildTree(), '(');
		boolean isUnique = (noInternalNodes==(ds.getNoOfSpecies()-1));
		
		if (!parameters.isNoLTR()){
			//TODO edit for forcing LRT computation
//			if (true)
			if (!isUnique)
			{
				int lowerBound = computeLowerBound(ds, parameters, bw);
				int upperBound = noInternalNodes-1;
				// compute least resolved tree
				solveLRT(ds, parameters, bw, lowerBound, upperBound);
				ds.computeLRTree();
			}
			else
			{
				System.out.println("BUILD tree is unique, skipping LeastResolvedTreeSolver");
				bw.write("#minimal resolved tree reconstruction time\n");
				bw.write("NULL\n");
			}
		}
	}
	
	public static void solveMCS(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw) throws IOException{
		// compute maximal consistent subset
		long time = System.currentTimeMillis();
		try {
			CplexMaximalConsistentSubsetSolver maximalConsistentSubsetSolver = new CplexMaximalConsistentSubsetSolver(ds, parameters);
			System.out.println("Running MaximalConsistentSubsetSolver");
			String maximalConsistentSubsetSolution = maximalConsistentSubsetSolver.solve();
			if (parameters.isVerbose()){
				printSolution(maximalConsistentSubsetSolution);
			}
		} catch (Exception e) {
			System.out.println("Warning: Could not compute maximal consistent subset");
			if (parameters.isVerbose()){
				e.printStackTrace();
			}
		}
		bw.write("#maximal consistent subset solving time\n");
		bw.write(String.valueOf((System.currentTimeMillis()-time))+"\n");
		bw.flush();
	}
	
	public static void greedyCS(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw) throws IOException{
		// compute greedy consistent subset
		long time = System.currentTimeMillis();
		try {
			GreedyConsistentSubsetSolver greedyConsistentSubsetSolver = new GreedyConsistentSubsetSolver(ds, parameters);
			System.out.println("Running GreedyConsistentSubsetSolver");
			String greedyConsistentSubsetSolution = greedyConsistentSubsetSolver.solve();
			if (parameters.isVerbose()){
				printSolution(greedyConsistentSubsetSolution);
			}
		} catch (Exception e) {
			System.out.println("Warning: Could not compute greedy consistent subset");
			if (parameters.isVerbose()){
				e.printStackTrace();
			}
		}
		bw.write("#greedy consistent subset solving time\n");
		bw.write(String.valueOf((System.currentTimeMillis()-time))+"\n");
		bw.flush();
	}
	
	public static String computeBuildTree(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw) throws IOException{
		// compute BUILD tree
		long time = System.currentTimeMillis();
		List<Triple> triples = new ArrayList<Triple>();
		for (int a=0; a<ds.getNoOfSpecies(); a++){
			for (int b=a+1; b<ds.getNoOfSpecies(); b++){
				for (int c=0; c<ds.getNoOfSpecies(); c++){
					if (a!=c && b!=c && ds.getConsistentTriple(a, b, c)){
						triples.add(new Triple(a, b, c));
					}
				}
			}
		}
		String buildTree = "none";
		try {
			System.out.println("Computing BUILD tree");
			Set<Integer> leaves = new HashSet<Integer>();
			for (int leaf=0; leaf<ds.getNoOfSpecies(); leaf++){
				leaves.add(leaf);
			}
			String support = String.valueOf(Utils.computeSupport(ds));
			buildTree = BuildAlgorithm.getTree(triples, leaves, ds.getSpeciesNames(), ds.getTriples())+support+";";
		} catch (Exception e) {
			e.printStackTrace();
		}
		bw.write("#BUILD tree reconstruction time\n");
		bw.write(String.valueOf((System.currentTimeMillis()-time))+"\n");
		bw.flush();
		
		return buildTree;
	}
	
	public static int computeLowerBound(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw) throws IOException{
		int lowerBound=0;
		try {
			CplexLeastResolvedTreeSizeLowerBoundSolver treeSizeLowerBoundSolver = new CplexLeastResolvedTreeSizeLowerBoundSolver(ds, parameters);
			System.out.println("Running CplexTreeSizeLowerBoundSolver");
			lowerBound = treeSizeLowerBoundSolver.solve();
		} catch (Exception e) {
			System.out.println("Warning: Could not compute lower bound for least resolved tree");
			if (parameters.isVerbose()){
				e.printStackTrace();
			}
		}
		return lowerBound;
	}
	
	public static void solveLRT(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, BufferedWriter bw, int lowerBound, int upperBound) throws IOException{
		// compute least resolved tree
		if (!parameters.isNoLTR()){
			long time = System.currentTimeMillis();
			try {
				CplexLeastResolvedTreeSolver leastResolvedTreeSolver = new CplexLeastResolvedTreeSolver(ds, parameters);
				System.out.println("Running LeastResolvedTreeSolver");
				String leastResolvedTreeSolution = leastResolvedTreeSolver.solve(lowerBound, upperBound);
				if (parameters.isVerbose()){
					printSolution(leastResolvedTreeSolution);
				}
			} catch (Exception e) {
				System.out.println("Warning: Could not compute least resolved tree");
				if (parameters.isVerbose()){
					e.printStackTrace();
				}
			}
			bw.write("#least resolved tree solving time\n");
			bw.write(String.valueOf((System.currentTimeMillis()-time))+"\n");
		}
	}

	public static void printSolution(String solution){
		System.out.println();
		System.out.println(solution);
		System.out.println();
	}
	
	private static void writeInfo(MultiGeneFamilySpeciesDataSet ds,
			Parameters parameters, BufferedWriter bw) throws IOException {
		
		bw.write("#no of species\n");
		bw.write(ds.getNoOfSpecies()+"\n");
		if (parameters.getInputType()!=Parameters.TRIPLE_INPUT){
			bw.write("#no of gene families\n");
			bw.write(ds.getGeneFamilies().size()+"\n");
			bw.write("#no of gene families containing triples\n");
			int n=0;
			for (GeneFamily gf : ds.getGeneFamilies()){
				if (gf.getTriples().size()>0){
					n++;
				}
			}
			bw.write(n+"\n");
		}
		bw.write("#no of extracted triples (distinct)\n");
		bw.write(ds.getNoOfDistinctTriples()+"\n");
		bw.write("#no of extracted triples (total)\n");
		bw.write(ds.getNoOfTriples()+"\n");
		if (ds.getNoOfEnrichedTriples()>0){
			bw.write("#no of enriched triples (total)\n");
			bw.write(ds.getNoOfEnrichedTriples()+"\n");
		}
		bw.write("#no of consistent triples\n");
		bw.write(ds.getNoOfConsistentTriples()+"\n");
		
		if (!ds.getOriginalTree().equals("")){
			bw.write("#original tree\n");
			bw.write(ds.getOriginalTree()+"\n");
		}

		bw.write("#BUILD tree\n");
		bw.write(ds.getBuildTree()+"\n");
		
		if (!parameters.isNoLTR()){
			bw.write("#least resolved tree\n");
			bw.write(ds.getLrTree()+"\n");
		}
		
		if (parameters.isVerbose()){
			bw.write("#triples\n");
		}
		double support = Double.NaN;
		double used = 0;
		double sum = 0;
		for (int a=0; a<ds.getNoOfSpecies(); a++){
			for (int b=a+1; b<ds.getNoOfSpecies(); b++){
				for (int c=0; c<ds.getNoOfSpecies(); c++){
					if (a!=c && b!=c && ds.getConsistentTriple(a, b, c)){
						double val=Double.NaN;
						used += ds.getTriple(a, b, c);
						sum += ds.getTriple(a, b, c) + ds.getTriple(a<c?a:c, a<c?c:a, b) + ds.getTriple(b<c?b:c, b<c?c:b, a);
						try {
							val = ((double)ds.getTriple(a, b, c))/((double)ds.getTriple(a, b, c)+ds.getTriple(a<c?a:c, a<c?c:a, b)+ds.getTriple(b<c?b:c, b<c?c:b, a));
						} catch (Exception e) {
						}
						if (parameters.isVerbose()){
							bw.write(a+","+b+"|"+c+"\t"+String.valueOf(val)+"\n");
						}
					}
				}
			}
		}
		if (sum > 0){
			support=used/sum;
		}
		bw.write("#support\n");
		bw.write(String.valueOf(support)+"\n");
	}
	
	public static void writeTrees(MultiGeneFamilySpeciesDataSet ds, Parameters parameters, boolean append) throws IOException{
		if (!ds.getOriginalTree().equals("")){
			writeFile(ds.getOriginalTree()+"\n", parameters.getOutput()+".org.tree", append);
		}
		
		if (parameters.isNoLTR()){
			writeFile(ds.getBuildTree()+"\n", parameters.getOutput()+".tree", append);
		}
		
		else {
			writeFile(ds.getLrTree()+"\n", parameters.getOutput()+".tree", append);
		}
	}
	
	public static void writeFile(String content, String file, boolean append) throws IOException{
		BufferedWriter bw = new BufferedWriter(new FileWriter(file, append));
		bw.write(content);
		bw.close();
	}
	
}
