package paraphylo.converter;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class Converter {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		String proteinorthoFile = "";
		String proteinorthoGraphFile = "";
		String blastDir = "";
		String alfVPFile = "";
		String alfTreeFile = "";
		String outFile = "";
		boolean bitscore = false;
		boolean evalue = false;
		double xpProbability = 0;
		int noiseType = 0;
		double noiseProbability = 0d;
		
		// create the command line parser
		CommandLineParser parser = new GnuParser();
		
		// create the Options
		Option h = new Option("h", "help", false, "print this message");
		Option p = new Option( "p", "proteinortho", true, "proteinortho output file" );
		Option g = new Option( "g", "proteinortho-graph", true, "proteinortho-graph file" );
		Option b = new Option( "b", "blast-dir", true, "directory with blast results" );
		Option s = new Option( "s", "bitscore", false, "store the bitscore");
		Option e = new Option( "e", "evalue", false, "store the evalue");
		Option a = new Option( "a", "alf-vp-archive", true, "VP.tgz file from ALF simulation" );
		Option r = new Option( "r", "alf-tree", true, "RealTree.nwk file from ALF simulation" );
		Option x = new Option( "x", "xp-probability", true, "probability that a xenolog simulated by ALF is mispredicted as paralog" );
		x.setType(Number.class);
		Option t = new Option( "t", "noise-type", true, "add noise to input data [default=0]\n\t0 - no noise\n\t1 - homologous noise\n\t2 - orthologous noise\n\t3 - paralogous noise\n\t4 - xenologous noise" );
		t.setType(Number.class);
		Option n = new Option( "n", "noise-probability", true, "probability of adding noise [default=0.0]" );
		n.setType(Number.class);
		Option o = new Option( "o", "output", true, "output filename" );
		
		Options options = new Options();
		options.addOption(h);
		options.addOption(p);
		options.addOption(g);
		options.addOption(b);
		options.addOption(s);
		options.addOption(e);
		options.addOption(a);
		options.addOption(r);
		options.addOption(x);
		options.addOption(t);
		options.addOption(n);
		options.addOption(o);

		try {
		    // parse the command line arguments
		    CommandLine line = parser.parse( options, args );
		    
		    if( line.hasOption( "help" ) ) {
		    	showHelp(options);
		    	return;
		    }
		    
		    if( line.hasOption( "proteinortho" ) && line.hasOption( "alf-vp-archive" )) {
	    		System.out.println( "ERROR: either a proteinortho file or an ALF VP.tgz file can be specified, but not both" );
	    		showHelp(options);
		    }

		    if( line.hasOption( "proteinortho" ) ) {
		    	proteinorthoFile = line.getOptionValue("proteinortho");
		    	if (proteinorthoFile == "" || !new File(proteinorthoFile).isFile()){
		    		System.out.println( "ERROR: proteinortho file '" + proteinorthoFile + "' not found" );
		    		showHelp(options);
		    	}
		    }
		    		    
		    if( line.hasOption( "proteinortho-graph" ) ) {
		    	proteinorthoGraphFile = line.getOptionValue("proteinortho-graph");
		    	if (proteinorthoFile == ""){
		    		System.out.println( "ERROR: proteinortho file required" );
		    		showHelp(options);
		    	}
		    	if (proteinorthoGraphFile == "" || !new File(proteinorthoGraphFile).isFile()){
		    		System.out.println( "ERROR: proteinortho-graph file '" + proteinorthoGraphFile + "' not found" );
		    		showHelp(options);
		    	}
		    }
		    		    
		    if( line.hasOption( "blast-dir" ) ) {
		    	blastDir = line.getOptionValue("blast-dir");
		    	if (blastDir == "" || !new File(blastDir).isDirectory()){
		    		System.out.println( "ERROR: blast results directory '" + blastDir + "' not found" );
		    		showHelp(options);
		    	}
		    }
		    
		    if( line.hasOption( "bitscore" ) ) {
		    	bitscore = true;
		    }
		    
		    if( line.hasOption( "evalue" ) ) {
		    	evalue = true;
		    }
		    
		    if( line.hasOption( "alf-vp-archive" ) ) {
		    	alfVPFile = line.getOptionValue("alf-vp-archive");
		    	if (alfVPFile == "" || !new File(alfVPFile).isFile()){
		    		System.out.println( "ERROR: ALF VP.tgz file '" + alfVPFile + "' not found" );
		    		showHelp(options);
		    	}
		    }
		    if( line.hasOption( "alf-tree" ) ) {
		    	alfTreeFile = line.getOptionValue("alf-tree");
		    	if (alfVPFile == ""){
		    		System.out.println( "ERROR: alf-vp-archive file required" );
		    		showHelp(options);
		    	}
		    	if (alfTreeFile == "" || !new File(alfTreeFile).isFile()){
		    		System.out.println( "ERROR: ALF RealTree.nwk file '" + alfTreeFile + "' not found" );
		    		showHelp(options);
		    	}
		    }
		    if( line.hasOption( "xp-probability" ) ) {
		    	xpProbability = ((Number)line.getParsedOptionValue("xp-probability")).doubleValue();
		    	xpProbability = Math.max(0, Math.min(xpProbability, 1d));
		    }
		    if( line.hasOption( "noise-type" ) ) {
		    	noiseType = ((Number)line.getParsedOptionValue("noise-type")).intValue();
		    }
		    if( line.hasOption( "noise-probability" ) ) {
		    	noiseProbability = ((Number)line.getParsedOptionValue("noise-probability")).doubleValue();
		    	noiseProbability = Math.max(0, Math.min(noiseProbability, 1d));
		    }		    
		    if( line.hasOption( "output" ) ) {
		    	outFile = line.getOptionValue("output");
		    	try {
					new File(outFile).getCanonicalPath();
				} catch (IOException ex) {
					System.out.println( "ERROR: no valid output file specified" );
					showHelp(options);
				}
		    }
		}
		catch( ParseException ex ) {
		    System.out.println( ex.getMessage() );
		    showHelp(options);
		}
		
		try {
			List<GeneFamily> families = new ArrayList<GeneFamily>(); 
			Map<String,Integer> geneFamilyMap = new HashMap<String, Integer>();
			String[] speciesNames = new String[0];
			String speciesTree = "";
			String output = "";
			if (proteinorthoFile!=""){
				speciesNames = ProteinorthoFileParser.parse(proteinorthoFile, geneFamilyMap, families);
				ProteinorthoGraphFileParser.parse(proteinorthoGraphFile, speciesNames, geneFamilyMap, families);
				if( blastDir!="" ) {
					BlastParser.parse(blastDir, geneFamilyMap, families);
					for (GeneFamily family : families){
						family.modify();
					}
				}
			}
			else if (alfVPFile!=""){
				speciesNames = ALFTreeFileParser.parseNames(alfTreeFile);
				speciesTree = ALFTreeFileParser.parseTree(alfTreeFile);
				ALFVPFileParser.parse(alfVPFile, speciesNames, geneFamilyMap, families, xpProbability);
			}
			int noSpecies = speciesNames.length;
			output = "#NoOfSpecies\n"+noSpecies+"\n";
			if (!speciesTree.equals("")){
				output += "#SpeciesTree\n";
				output += speciesTree+"\n";
			}			
			output += "#SpeciesNames\n";
			for (String speciesName : speciesNames){
				output += speciesName + "\t";
			}
			if (noiseType>0){
				for (GeneFamily family : families){
					family.addNoise(new Random(), noiseType, noiseProbability, speciesNames);
				}
			}
			output += "\n\n";
			int counter = 1;
			for (GeneFamily family : families){
				output+=family.output(counter, bitscore, evalue);
				counter++;
			}
			
			if( outFile != "" ) {
				BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
				bw.write(output);
				bw.close();
			}
			else{
				System.out.println(output);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	public static <T extends Option> void showHelp(Options options){
		// automatically generate the help statement
		HelpFormatter formatter = new HelpFormatter();
		formatter.setOptionComparator(new Comparator<T>() {
			private static final String OPTS_ORDER = "hpgbsearxtno";
		    public int compare(T o1, T o2) {
		        return OPTS_ORDER.indexOf(o1.getOpt()) - OPTS_ORDER.indexOf(o2.getOpt());
		    }
		});
		formatter.printHelp( "java -jar ParaPhyloConverter -p <proteinortho file> -g <proteinortho-graph file> [OPTIONS]\n+" +
				"or\n+" +
				"java -jar ParaPhyloConverter -a <ALF VP.tgf file> -r <ALF RealTree.nwk file> [OPTIONS]", options );
		System.exit(0);
	}

}
